import sys
import os

sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

from typing import List

from .nasdaq_lists import country_acronyms, repo_codes, sp500_codes, income_expenditure_codes, economic_indicator_codes, unemployment_codes, interest_rate_codes, gdp_codes
import pandas as pd
import requests
import io
session = requests.session()
class Nasdaq:
    def __init__(self):
        self.api_key = os.environ.get("YOUR_NASDAQ_KEY")
        self.base_url = "https://data.nasdaq.com/api/v3/datasets/"



    def bigmac(self):
        """Download Big Mac valuation data per country.

        Args:
            country_acronyms (List[str]): List of country acronyms to download data for.

        Returns:
            A csv file in the corresponding directory.
        """
        session = requests.Session()

        for country in country_acronyms:
            r = session.get(f"{self.base_url}ECONOMIST/BIGMAC_{country}.csv?&api_key={self.api_key}")

            # Check if the request was successful
            if r.status_code == 200:
                # Create a file-like object from the response content
                file_like_object = io.BytesIO(r.content)

                df = pd.read_csv(file_like_object)
                directory = 'files/nasdaq/bigmac/'

                # Create the directory if it doesn't exist
                if not os.path.exists(directory):
                    os.makedirs(directory)

                # Save the DataFrame to a separate CSV file for each country
                filename = f'{directory}{country}bigmac_data.csv'
                df.to_csv(filename, index=False)
                print(f"Saved data for {country} to {filename}")
            else:
                print(f"Error downloading CSV for {country}: {r.status_code}")

    def inflation(self):
        """Download inflation data across the globe.

        Args:
            country_acronyms (List[str]): List of country acronyms to download data for.

        Returns:
            A csv file in the corresponding directory.
        """
        session = requests.Session()

        for country in country_acronyms:
            r = session.get(f"{self.base_url}RATEINF/CPI_{country}.csv?&api_key={self.api_key}")

            # Check if the request was successful
            if r.status_code == 200:
                # Create a file-like object from the response content
                file_like_object = io.BytesIO(r.content)

                df = pd.read_csv(file_like_object)
                directory = 'files/nasdaq/inflation/'

                # Create the directory if it doesn't exist
                if not os.path.exists(directory):
                    os.makedirs(directory)

                # Save the DataFrame to a separate CSV file for each country
                filename = f"{directory}{country}inflation.csv"
                df.to_csv(filename, index=False)
                print(f"Saved data for {country} to {filename}")
            else:
                print(f"Error downloading CSV for {country}: {r.status_code}")

    def repo(self):
        """Download repo data from the Federal Reserve.

        Args:
            repo_codes (List[str]): List of repo codes to download data for.

        Returns:
            A csv file in the corresponding directory.
        """
        session = requests.Session()

        for repo_code in repo_codes:
            r = session.get(f"{self.base_url}FRED/{repo_code}.csv?&api_key={self.api_key}")

            # Check if the request was successful
            if r.status_code == 200:
                # Create a file-like object from the response content
                file_like_object = io.BytesIO(r.content)

                df = pd.read_csv(file_like_object)
                directory = 'files/nasdaq/repo/'

                # Create the directory if it doesn't exist
                if not os.path.exists(directory):
                    os.makedirs(directory)

                # Save the DataFrame to a separate CSV file for each repo code
                filename = f"{directory}{repo_code}_repo.csv"
                df.to_csv(filename, index=False)
                print(f"Saved data for {repo_code} to {filename}")
            else:
                print(f"Error downloading CSV for {repo_code}: {r.status_code}")



    def sp500(self):
        """Download SP500 macro data from the Federal Reserve.

        Args:
            sp500_codes (List[str]): List of SP500 codes to download data for.

        Returns:
            A csv file in the corresponding directory.
        """
        session = requests.Session()

        for sp500_code in sp500_codes:
            r = session.get(f"{self.base_url}MULTPL/{sp500_code}.csv?&api_key={self.api_key}")

            # Check if the request was successful
            if r.status_code == 200:
                # Create a file-like object from the response content
                file_like_object = io.BytesIO(r.content)

                df = pd.read_csv(file_like_object)
                directory = 'files/nasdaq/sp_500/'

                # Create the directory if it doesn't exist
                if not os.path.exists(directory):
                    os.makedirs(directory)

                # Save the DataFrame to a separate CSV file for each SP500 code
                filename = f"{directory}{sp500_code}.csv"
                df.to_csv(filename, index=False)
                print(f"Saved data for {sp500_code} to {filename}")
            else:
                print(f"Error downloading CSV for {sp500_code}: {r.status_code}")

    def income_expenditures(self):
        """Download income expenditure data from the Federal Reserve.

        Args:
            income_expenditure_codes (List[str]): List of income expenditure codes to download data for.

        Returns:
            A csv file in the corresponding directory.
        """
        session = requests.Session()

        for income_expenditure_code in income_expenditure_codes:
            r = session.get(f"{self.base_url}FRED/{income_expenditure_code}.csv?&api_key={self.api_key}")

            # Check if the request was successful
            if r.status_code == 200:
                # Create a file-like object from the response content
                file_like_object = io.BytesIO(r.content)

                df = pd.read_csv(file_like_object)
                directory = 'files/nasdaq/income_expenditures/'

                # Create the directory if it doesn't exist
                if not os.path.exists(directory):
                    os.makedirs(directory)

                # Save the DataFrame to a separate CSV file for each income expenditure code
                filename = f"{directory}{income_expenditure_code}.csv"
                df.to_csv(filename, index=False)
                print(f"Saved data for {income_expenditure_code} to {filename}")
            else:
                print(f"Error downloading CSV for {income_expenditure_code}: {r.status_code}")

    def economic_indicators(self):
        """Download economic indicator data from the Federal Reserve.

        Args:
            economic_indicator_codes (List[str]): List of economic indicator codes to download data for.

        Returns:
            A csv file in the corresponding directory.
        """
        session = requests.Session()

        for economic_indicator_code in economic_indicator_codes:
            r = session.get(f"{self.base_url}FRED/{economic_indicator_code}.csv?&api_key={self.api_key}")

            # Check if the request was successful
            if r.status_code == 200:
                # Create a file-like object from the response content
                file_like_object = io.BytesIO(r.content)

                df = pd.read_csv(file_like_object)
                directory = 'files/nasdaq/economic_indicators/'

                # Create the directory if it doesn't exist
                if not os.path.exists(directory):
                    os.makedirs(directory)

                # Save the DataFrame to a separate CSV file for each economic indicator code
                filename = f"{directory}{economic_indicator_code}.csv"
                df.to_csv(filename, index=False)
                print(f"Saved data for {economic_indicator_code} to {filename}")
            else:
                print(f"Error downloading CSV for {economic_indicator_code}: {r.status_code}")


    def unemployment(self):
        """Return unemeployment data from the FED
        
        
        Returns: A csv file in the corresponding directory.
        """
        for unemployment_code in unemployment_codes:
            r = session.get(f"{self.base_url}FRED/{unemployment_code}.csv?&api_key={self.api_key}")



            # Check if the request was successful
            if r.status_code == 200:
                # Create a file-like object from the response content
                file_like_object = io.BytesIO(r.content)

                df = pd.read_csv(file_like_object)
                directory = 'files/nasdaq/unemployment/'

                # Create the directory if it doesn't exist
                if not os.path.exists(directory):
                    os.makedirs(directory)
                # Save the DataFrame to a separate CSV file for each country

                filename = f"{directory}{unemployment_code}_repo.csv"
                df.to_csv(filename, index=False)
                print(f"Saved data for {unemployment_code} to {filename}")

            else:
                print(f"Error downloading CSV for {unemployment_code}: {r.status_code}")



    def interest_rates(self):
        """Download interest rate data for the specified interest rate codes.

        Args:
            interest_rate_codes (List[str]): List of interest rate codes to download data for.

        Returns:
            None
        """
        session = requests.Session()

        for interest_rate_code in interest_rate_codes:
            r = session.get(f"{self.base_url}FRED/{interest_rate_code}.csv?&api_key={self.api_key}")

            # Check if the request was successful
            if r.status_code == 200:
                # Create a file-like object from the response content
                file_like_object = io.BytesIO(r.content)

                df = pd.read_csv(file_like_object)
                directory = 'files/nasdaq/interest_rates/'

                # Create the directory if it doesn't exist
                if not os.path.exists(directory):
                    os.makedirs(directory)

                # Save the DataFrame to a separate CSV file for each interest rate code
                filename = f"{directory}{interest_rate_code}_repo.csv"
                df.to_csv(filename, index=False)
                print(f"Saved data for {interest_rate_code} to {filename}")
            else:
                print(f"Error downloading CSV for {interest_rate_code}: {r.status_code}")

    def gdp(self):
        """Download gross domestic product (GDP) data for the specified GDP codes.

        Args:
            gdp_codes (List[str]): List of GDP codes to download data for.

        Returns:
            None
        """
        session = requests.Session()

        for gdp_code in gdp_codes:
            r = session.get(f"{self.base_url}FRED/{gdp_code}.csv?&api_key={self.api_key}")

            # Check if the request was successful
            if r.status_code == 200:
                # Create a file-like object from the response content
                file_like_object = io.BytesIO(r.content)

                df = pd.read_csv(file_like_object)
                directory = 'files/nasdaq/gdp/'

                # Create the directory if it doesn't exist
                if not os.path.exists(directory):
                    os.makedirs(directory)

                # Save the DataFrame to a separate CSV file for each GDP code
                filename = f"{directory}{gdp_code}_repo.csv"
                df.to_csv(filename, index=False)
                print(f"Saved data for {gdp_code} to {filename}")
            else:
                print(f"Error downloading CSV for {gdp_code}: {r.status_code}")




from dataclasses import dataclass, asdict
from datetime import datetime

import pandas as pd

@dataclass
class JoblessClaimsData:
    date: datetime
    number: int
    percent_change: float


    def as_dataframe(self) -> pd.DataFrame:
        """Converts the object to a pandas DataFrame."""
        df = pd.DataFrame([asdict(self)])
        df.columns = df.columns.str.lower()
        return df
    
    def as_dict(self) -> dict:
        """Converts the object to a dictionary with lowercase keys."""
        return {k.lower(): v for k, v in asdict(self).items()}

    def insert_into_pg(self, conn):
        cursor = conn.cursor()
        insert_query = """INSERT INTO jobless_claims (date, number, percent_change) VALUES (%s, %s, %s);"""
        cursor.execute(insert_query, (self.date, self.number, self.percent_change))
        conn.commit()




import sys
import os

sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
import requests



import pandas as pd
from .models import AuctionResult, FXSwaps, TimeSeries, AsOfDates, TimeSeriesData, SecuredReferenceRates, RepoOperations, SecuritiesLending
session = requests.session()

today_str = '2023-10/29'

class FedNewyork:
    def __init__(self):
        self.base_url = "https://markets.newyorkfed.org/api/"


    def agency_mbs_search(self, start_date="2023-01-01", end_date=today_str):
        """Search for AMBS operations out of the Federal Reserve of New York"""

        r = session.get(self.base_url + f"ambs/all/results/summary/search.json?startDate={start_date}&endDate={today_str}").json()
        ambs = r['ambs']
        auctions = ambs['auctions']
        if auctions is not None:
            data = AuctionResult(auctions)
            return data
        else:
            return None
    

    def agency_mbs_count(self, number=10):
        """Return AMBS transactions by count."""
        r = session.get(self.base_url + f"ambs/all/results/details/last/{number}.json").json()
        ambs = r['ambs']
        auctions = ambs['auctions']
        if auctions is not None:
            data = AuctionResult(auctions)
            return data
        else:
            return None
        

    def liquidity_swaps_latest(self):
        """Get the latest central bank liquidity swap data."""
        r = session.get(self.base_url + f"fxs/all/latest.json").json()
        fxSwaps = r['fxSwaps']
        operations = fxSwaps['operations']
        if operations is not None:
            data = FXSwaps(operations)
            return data
        else:
            return "No recent data found."
        
    def liquidity_swaps_count(self, number=50):
        """Get the latest central bank liquidity swap data."""
        r = session.get(self.base_url + f"fxs/usdollar/last/{number}.json").json()
        fxSwaps = r['fxSwaps']
        operations = fxSwaps['operations']
        if operations is not None:
            data = FXSwaps(operations)
            return data
        else:
            return "No recent data found."
        

    def liquidity_swaps_search(self, start_date = "2023-01-01", end_date = today_str, type="trade", counterparties='japan,europe'):
        """Search for liquidity swaps between a custom date range.
        
        Arguments:
          >>> start_date: a date in YYYY-MM-DD format to start from
          >>> end_date: a date in YYYY-MM-DD to end on. defaults to today.
          >>> type: type of information to return. trade or maturity.
          >>> counterparties: list of counterparties. default: europe, japan
        """

        

        r = session.get(self.base_url + f"fxs/all/search.json?startDate={start_date}&endDate={end_date}&dateType={type}&counterparties={counterparties}").json()
        fxSwaps = r['fxSwaps']
        operations = fxSwaps['operations']
        if operations is not None:
            data = FXSwaps(operations)
            return data
        else:
            return None
        

    def get_fed_counterparties(self):
        """Returns the current counterparties to the Federal Reserve."""
        r = session.get(self.base_url + "fxs/list/counterparties.json").json()
        fxSwaps = r['fxSwaps']
        counterparties = fxSwaps['counterparties']
        if counterparties is not None:
            return counterparties


    def get_as_of_dates(self):
        """Returns a list of dates to query the FED API with."""
        r = session.get("https://markets.newyorkfed.org/api/pd/list/asof.json").json()
        pdd = r['pd']
        as_of_dates = pdd['asofdates']
        if as_of_dates is not None:
            data = AsOfDates(as_of_dates)
            return data
        else:
            return None
        
    def get_timeseries(self):
            """Returns the timeseries data to query the FED API"""

            r = requests.get("https://markets.newyorkfed.org/api/pd/list/timeseries.json").json()
            pdd = r['pd']
            timeseries = pdd['timeseries']
            if timeseries is not None:
                data = TimeSeries(timeseries)

                
                return data
            else:
                return None


    def get_timeseries_data(self, timeseries):
        """Use timeseries codes to query the FED API."""
        
        timeseries_data = requests.get(f"https://markets.newyorkfed.org/api/pd/get/{timeseries}.json").json()
        pdd = timeseries_data['pd']
        timeseries = pdd['timeseries']
        if timeseries is not None:
            data = TimeSeriesData(timeseries)
            return data
        else:
            return None
    
    def reference_rates(self, type):
        """Returns all unsecured central bank rates globally.
        
        Arguments:
        >>> rate_type: secured or unsecured
        """
        r = session.get(f"https://markets.newyorkfed.org/api/rates/{type}/all/latest.json").json()
        refrates = r['refRates']

        if refrates is not None:
            data = SecuredReferenceRates(refrates)
            return data
        else:
            return None
        

        

    def rates_search(self, start_date="2023-01-01", end_date=today_str):
        """Search reference rates between a given time range."""
        r = session.get(self.base_url + f"rates/all/search.json?startDate={start_date}&endDate={end_date}").json()
        refrates = r['refRates']
        if refrates is not None:
            data = SecuredReferenceRates(refrates)
            return data
        else:
            return None
        

    def repo_operations_search(self, start_date="2023-01-01", end_date=today_str):
        """Search by date for repo operations out of the FED."""

        r = session.get(f"https://markets.newyorkfed.org/api/rp/results/search.json?startDate={start_date}&endDate={end_date}&securityType=mb").json()
        repo = r['repo']
        operations = repo['operations']
        if operations is not None:
            data = RepoOperations(operations)
            return data
        else:
            return None
        

    def repo_latest(self):
        """Get the latest repo operations from the FED's discount window."""

        r = session.get("https://markets.newyorkfed.org/api/rp/all/all/results/latest.json").json()

        repo = r['repo']
        operations = repo['operations']
        if operations is not None:
            data = RepoOperations(operations)

            return data
        else:
            return None
        

    def repo_propositions(self):
        """Check all repo & reverse repo operations out of the FED."""
        propositions = session.get("https://markets.newyorkfed.org/api/rp/reverserepo/propositions/search.json").json()

        repo = propositions['repo']
        operations = repo['operations']
        data = []
        for operation in operations:
            operation_id = operation['operationId']
            operation_date = operation['operationDate']
            operation_type = operation['operationType']
            note = operation['note']
            total_amt_accepted = operation['totalAmtAccepted']
            
            data.append({
                'Operation ID': operation_id,
                'Operation Date': operation_date,
                'Operation Type': operation_type,
                'Note': note,
                'Total Amount Accepted': total_amt_accepted
            })

        df = pd.DataFrame(data)
        return df


    def securities_lending_search(self, start_date="2023-01-01", end_date=today_str):
        """Search securities lending operations out of the FED."""
        sec_lending = session.get(f"https://markets.newyorkfed.org/api/seclending/all/results/summary/search.json?startDate={start_date}&endDate={end_date}").json()

        seclending = sec_lending['seclending']
        operations = seclending['operations']
        if operations is not None:
            data = SecuritiesLending(operations)
            return data
        else:
            return None
                





from datetime import datetime, timezone
import pytz

import pandas as pd

class RSI:
    def __init__(self, data, ticker):
        self.as_dataframe = None
        if 'results' in data:
            results = data['results']
            values = results.get('values', None)
            self.ticker=ticker
            self.rsi_value = [i.get('value', None) for i in values] if values and isinstance(values, list) and any('value' in i for i in values) else None
            self.rsi_timestamp = [i.get('timestamp', None) for i in values] if values and isinstance(values, list) and any('value' in i for i in values) else None
            if self.rsi_timestamp is not None:
                # Convert timestamps to human-readable Eastern Time
                self.rsi_timestamp = self.convert_to_human_readable(self.rsi_timestamp)

                self.data_dict = { 

                    'ticker': ticker,

                    'rsi_value': self.rsi_value,

                    'rsi_timestamp': self.rsi_timestamp
                }


                self.as_dataframe = pd.DataFrame(self.data_dict)
            
    def convert_to_human_readable(self, timestamps):
        human_readable_times = []
        eastern = pytz.timezone('US/Eastern')
        if timestamps is not None:
            for ts in timestamps:
                if ts is not None:
                    try:
                        ts /= 1000  # Convert from milliseconds to seconds
                        dt_utc = datetime.fromtimestamp(ts, timezone.utc)
                        dt_eastern = dt_utc.astimezone(eastern)
                        human_readable_times.append(dt_eastern.strftime('%Y-%m-%d %H:%M:%S'))
                    except Exception as e:
                        print(f"Failed to convert timestamp {ts}: {e}")
                else:
                    human_readable_times.append(None)  # or some default value
            return human_readable_times
    

    
    



from datetime import datetime, timezone
import pytz

import pandas as pd

class EMA:
    def __init__(self, data, ticker):
        results = data['results']
        values = results.get('values')

        self.ema_value = [i.get('value') for i in values]
        self.ema_timestamp = [i.get('timestamp') for i in values]

        # Convert timestamps to human-readable Eastern Time
        self.ema_timestamp = self.convert_to_human_readable(self.ema_timestamp)

        self.data_dict = { 
            'ticker': ticker,

            'ema_value': self.ema_value,

            'ema_timestamp': self.ema_timestamp
        }


        self.as_dataframe = pd.DataFrame(self.data_dict)
        
    def convert_to_human_readable(self, timestamps):
        human_readable_times = []
        eastern = pytz.timezone('US/Eastern')
        for ts in timestamps:
            if ts is not None:
                try:
                    ts /= 1000  # Convert from milliseconds to seconds
                    dt_utc = datetime.fromtimestamp(ts, timezone.utc)
                    dt_eastern = dt_utc.astimezone(eastern)
                    human_readable_times.append(dt_eastern.strftime('%Y-%m-%d %H:%M:%S'))
                except Exception as e:
                    print(f"Failed to convert timestamp {ts}: {e}")
            else:
                human_readable_times.append(None)  # or some default value
        return human_readable_times
    


class SMA:
    def __init__(self, data, ticker):
        results = data['results']
        values = results.get('values')

        self.sma_value = [i.get('value') for i in values]
        self.sma_timestamp = [i.get('timestamp') for i in values]

        # Convert timestamps to human-readable Eastern Time
        self.sma_timestamp = self.convert_to_human_readable(self.sma_timestamp)

        self.data_dict = { 
            'ticker': ticker,
            'sma_value': self.sma_value,

            'sma_timestamp': self.sma_timestamp
        }


        self.as_dataframe = pd.DataFrame(self.data_dict)
        
    def convert_to_human_readable(self, timestamps):
        human_readable_times = []
        eastern = pytz.timezone('US/Eastern')
        for ts in timestamps:
            if ts is not None:
                try:
                    ts /= 1000  # Convert from milliseconds to seconds
                    dt_utc = datetime.fromtimestamp(ts, timezone.utc)
                    dt_eastern = dt_utc.astimezone(eastern)
                    human_readable_times.append(dt_eastern.strftime('%Y-%m-%d %H:%M:%S'))
                except Exception as e:
                    print(f"Failed to convert timestamp {ts}: {e}")
            else:
                human_readable_times.append(None)  # or some default value
        return human_readable_times